package org.richfaces.photoalbum.util;

import java.util.List;

import javax.persistence.EntityManager;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.core.Events;
import org.richfaces.photoalbum.domain.Comment;
import org.richfaces.photoalbum.domain.User;
import org.richfaces.photoalbum.service.Constants;
import org.richfaces.photoalbum.service.IImageAction;

@Scope(ScopeType.SESSION)
@Name("sessionListener")
@Startup
public class SessionListener {

	@In(required=false) User user;
	
	@In IImageAction imageAction;
	
	@In(value="entityManager") EntityManager em;
	
	@Destroy @Transactional @Observer("org.jboss.seam.sessionExpired")
	public void onDestroy(){
		if(user.getId() != null && !user.isPreDefined()){
			user = em.merge(user);
			List<Comment> userComments = imageAction.findAllUserComments(user);
			for(Comment c : userComments){
				em.remove(c);
			}
			em.remove(user);
			em.flush();
			Events.instance().raiseEvent(Constants.USER_DELETED_EVENT, user);
		}
	}
}
